#!/bin/awk -f
#
# Convert from K-NET or KiK-net format to SAC ASCII format
#
# 2005/03/30 : written by Reiji Kobayashi
#

#
# function: month+day 
#    --> julian day (cumulative days from the initial day of the year)
#
function md2jd(yr,mn,dy,jd){
    split("31,28,31,30,31,30,31,31,30,31,30,31",md,",");
    jd=0;

    if(yr < 1000 || yr > 3000) {
	printf("year=%d is real? \n", yr);
	exit;
    }
    if(mn < 1 || mn > 12) { 
	printf("month=%d is out of range 1-12 \n", mn);
	exit;
    }

    if(( yr%4 == 0 && yr%100 != 0 ) || yr%400 == 0 ) ++md[2];

    if(dy < 1 || dy > md[mn]) { 
	printf("day=%d is out of range 1-%2d \n", dy, md[mn]);
	exit;
    }

    for (im = 1; im < mn; im++){
	jd += md[im];
    }

    jd += dy;
    return jd;

}

#
# function: write data in sac ascii format
#
function wsac(delta,b,omarker,stlat,stlon,stelv,evlat,evlon,evdep,evmag,cmpaz,cmpinc,evyear,evjday,evhour,evmin,evsec,evmsec,ndata,station,eventNo,comp,net,wav,fil,filsac){
    fnon=-12345.0;
    inon=-12345;
    snon=-12345;
#    ffmt="%15.7f%15.7f%15.7f%15.7f%15.7f\n";
    ffmt="%15.7e%15.7e%15.7e%15.7e%15.7e\n";
    ifmt="%10d%10d%10d%10d%10d\n";
    sfmt="%-8.8s%-8.8s%-8.8s\n";
    printf(ffmt,delta,fnon,fnon,fnon,fnon) > filsac;
    printf(ffmt,b,fnon,omarker,fnon,fnon) > filsac;
    printf(ffmt,fnon,fnon,fnon,fnon,fnon) > filsac;
    printf(ffmt,fnon,fnon,fnon,fnon,fnon) > filsac;
    printf(ffmt,fnon,fnon,fnon,fnon,fnon) > filsac;
    printf(ffmt,fnon,fnon,fnon,fnon,fnon) > filsac;
    printf(ffmt,fnon,stlat,stlon,stelv,fnon) > filsac;
    printf(ffmt,evlat,evlon,fnon,evdep,evmag) > filsac;
    printf(ffmt,fnon,fnon,fnon,fnon,fnon) > filsac;
    printf(ffmt,fnon,fnon,fnon,fnon,fnon) > filsac;
    printf(ffmt,fnon,fnon,fnon,fnon,fnon) > filsac;
    printf(ffmt,fnon,fnon,cmpaz,cmpinc,fnon) > filsac;
    printf(ffmt,fnon,fnon,fnon,fnon,fnon) > filsac;
    printf(ffmt,fnon,fnon,fnon,fnon,fnon) > filsac;
    printf(ifmt,evyear,evjday,evhour,evmin,evsec) > filsac;
    printf(ifmt,evmsec,6,inon,inon,ndata) > filsac;
    printf(ifmt,inon,inon,inon,inon,inon) > filsac;
    printf(ifmt,01,inon,inon,inon,inon) > filsac;
    printf(ifmt,inon,inon,40,inon,inon) > filsac;
    printf(ifmt,inon,inon,inon,inon,inon) > filsac;
    printf(ifmt,inon,inon,inon,inon,inon) > filsac;
    printf(ifmt,1,inon,1,1,inon) > filsac;
    printf("%-8.8s%-16.16s\n",station,eventNo) > filsac;
    printf(sfmt,snon,snon,snon) > filsac;
    printf(sfmt,snon,snon,snon) > filsac;
    printf(sfmt,snon,snon,snon) > filsac;
    printf(sfmt,snon,snon,snon) > filsac;
    printf(sfmt,snon,snon,snon) > filsac;
    printf(sfmt,snon,snon,comp) > filsac;
    printf(sfmt,net,snon,snon) > filsac;
    for(idata=1;idata<=ndata;++idata){
	print b+(idata-1)*delta, wav[idata] > fil;
	printf("%15.7f",wav[idata]) > filsac;
        if ( ( ((idata)%5) == 0) && (idata > 0) ) printf("\n") > filsac;
    }
    if ( ( ((idata)%5) != 0) && (idata > 0) ) printf("\n") > filsac;

}

#
# read source parameters from a file
#
BEGIN{
    getline < filsrc;
    evyear=$1+0;
    evmonth=$2+0;
    evday=$3+0;
    evjday=md2jd(evyear,evmonth,evday);
    evhour=$4+0;
    evmin=$5+0;
    evsec=$6+0;
    evmsec=$7+0;
    evlat=$8+0;
    evlon=$9+0;
    evdep=$10+0;
    evmag=$11+0;
}

{
    if(NR<=17){
       if(NR==6){
	   stname=$3;
       }
       if(NR==7){
	   stlat=$3+0;
       }
       if(NR==8){
	   stlon=$3+0;
       }
       if(NR==9){
	   stelv=$3+0;
       }
       if(NR==10){
	   bgdate=$3;
	   bgtime=$4;
	   split(bgdate,bd,"/");
	   bgyear=bd[1]+0;
	   bgmonth=bd[2]+0;
	   bgday=bd[3]+0;
	   split(bgtime,bt,":");
	   bghour=bt[1]+0;
	   bgmin=bt[2]+0;
	   bgsec=bt[3]-15.;
	   bgjday=md2jd(bgyear,bgmonth,bgday);
	   b=(bgjday-evjday)*(24*3600)+(bghour-evhour)*3600+(bgmin-evmin)*60+(bgsec-evsec)+(0.-evmsec)*0.001;
#	   print "b=", b;
	   Omarker=0.0;
       }
       if(NR==11){
	   samp=$3+0;
	   gsub(/Hz/,"",samp);
	   samp=samp+0.;
	   delta=1./samp;
       }
       if(NR==12){
	   npts=$3*samp;
       }
       if(NR==13){
	   kkik=substr(stname,4,1);
	   if(kkik=="H"){
	       net="KiK-net"
	       if($2=="1"){comp="NS1";cmpaz=0.0;cmpinc=90.0;}
	       if($2=="2"){comp="EW1";cmpaz=90.0;cmpinc=90.0;}
	       if($2=="3"){comp="UD1";cmpaz=0.0;cmpinc=0.0;}
	       if($2=="4"){comp="NS2";cmpaz=0.0;cmpinc=90.0;}
	       if($2=="5"){comp="EW2";cmpaz=90.0;cmpinc=90.0;}
	       if($2=="6"){comp="UD2";cmpaz=0.0;cmpinc=0.0;}
	   }else{
	       net="K-NET";
	       if($2=="N-S"){comp="NS";cmpaz=0.0;cmpinc=90.0;}
	       if($2=="E-W"){comp="EW";cmpaz=90.0;cmpinc=90.0;}
	       if($2=="U-D"){comp="UD";cmpaz=0.0;cmpinc=0.0;}
	   }
#	   print net, $2, comp, cmpaz, cmpinc;
       }
       if(NR==14){
	   scale=$3;
	   split(scale,idx,/\(gal\)\//);
	   scale=idx[1]/idx[2];
       }
    }else{
	for (i=1; i<=NF; ++i){
	    idat = (NR-18)*8+i;
	    wav[idat]=$i*scale;
	}
    }
}

END{
    wsac(delta,b,omarker,stlat,stlon,stelv,evlat,evlon,evdep,evmag,cmpaz,cmpinc,evyear,evjday,evhour,evmin,evsec,evmsec,npts,stname,evname,comp,net,wav,filtmp,filsac);
}
